complete -c sqv -n "__fish_use_subcommand" -l keyring -d 'A keyring.  Can be given multiple times.'
complete -c sqv -n "__fish_use_subcommand" -s n -l signatures -d 'The number of valid signatures to return success.  Default: 1'
complete -c sqv -n "__fish_use_subcommand" -l not-before -d 'Consider signatures created before TIMESTAMP as invalid.  If a date is given, 00:00:00 is used for the time. 
[default: no constraint]'
complete -c sqv -n "__fish_use_subcommand" -l not-after -d 'Consider signatures created after TIMESTAMP as invalid.  If a date is given, 23:59:59 is used for the time. 
[default: now]'
complete -c sqv -n "__fish_use_subcommand" -s v -l verbose -d 'Be verbose.'
complete -c sqv -n "__fish_use_subcommand" -s h -l help -d 'Prints help information'
complete -c sqv -n "__fish_use_subcommand" -s V -l version -d 'Prints version information'
